/*

An example of license ...

This source file is part of Scol
For the latest info, see http://www.scolring.org

Copyright (c) 2010 Iri <iri@irizone.net>

This source is under the terms of the Scol License. See the COPYING included file
for more informations.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.

For others informations, please contact us from http://www.scolring.org/

*/

/** \file main.c
 * \brief fichier main
 * contient le code des globales de l'API
 * \author iri <iri-@-irizone.net>
 * \date novembre 2010
 */

#include "main.h"


/*
    Définition de deux variables globales établissant la liaison avec les structures partagées du noyau
    la structure cbmachine n'est toutefois définie que dans la version MS Windows
*/
#if defined _WIN32 || defined __WIN32__
cbmachine ww;
#endif
mmachine  mm;



/* Destruction implicte de l'objet Scol s'il existe */
int ObjCurlDestroy (mmachine m, int handsys, int mobj)
{
    CURL * c;

    MMechostr(MSKDEBUG, "ObjCurlDestroy: entering ...\n");

    c = (CURL *) MMfetch (m, MTOP (mobj), OBJCURL_HANDLE);
    if (c == NULL)
    {
        MMechostr(MSKDEBUG, "ObjCurlDestroy : object already destroyed\n");
        return 0;
    }
    /*curl_easy_cleanup (c);*/

    MMstore (m, MTOP (mobj), OBJCURL_HANDLE, (int) NULL);
    MMechostr(MSKDEBUG, "ObjCurlDestroy: object has been destroyed\n");

    return 0;
}



/** \brief Définition de l'API
 *
 * \param
 * \return sans intérêt ici
 *
 * Quatre tableaux qui sont présentés à la fonction 'SCOLinitEXAMPLEclass'
 * Les éléments définis dans ce preimer tableau seront alors accessibles depuis n'importe quel script Scol.
 *
 */
char* example_name[EXAMPLE_PKG_NB] =
{
    /* nouveau type Scol */
    "ObjCurl",

    /* nouvelles fonctions Scol */
    "example_getFile",    /* fonction de l'API appelable depuis l'extérieur */
    "example_newUrl",
    "example_delGetUrl"
};

/** \brief Définition des fonctions internes
 *
 * Ce tableau donne la correspondance des fonctions Scol vers le code interne.
 * À ce sujet, en C, les flags peuvent poser problèmes. Le code présenté ici n'est pas valide
 * ANSI C (une option -ansi de gcc provoquera un warning non bloquant).
 *
 * Dans le code source existant, il existe parfois ceci :
 * \code #define bullshit int (__cdecl *)(struct Mmachine *)
 * (bullshit) (1*2)\endcode
 * Cette solution fonctionne dans la plupart des cas mais n'est pas garantie (outre le fait
 * que gcc lancera au mieux un warning , probablement une erreur à moins d'être très laxiste
 * sur ces options.
 */
int (*example_fun[EXAMPLE_PKG_NB]) (mmachine m) =
{
    NULL,

    sc_getFile,   /* nom interne où le code de cette fonction est développé */
    sc_newGetUrl,
    sc_delGetUrl
};

/** \brief Nombre d'arguments ou type d'objets
 *
 * Lorsqu'il s'agit d'une fonction Scol, le nombre exact d'arguments doit être indiqué.
 * Sinon, divers flags sont possibles. Les plus courants sont TYPVAR pour une valeur
 * numérique et TYPTYPE pour un type objet Scol. Les autres sont listés sur le tutoriel
 * ou, de façon moins expliquée, sur scol_plugin_xxx.h
 */
int example_narg[EXAMPLE_PKG_NB] =
{
    TYPTYPE,

    2,      /* nombre d'argument attendu par cette fonction */
    5,
    1
};

/** \brief Prototypage Scol
 *
 * Les prototypes Scol des fonctions doivent correspondre avec le nmbre d'arguments notés au
 * tableau 3. Dans le cas contraire, il y aura une erreur à l'exécution lors de l'appel de la fonction
 */
char* example_type[EXAMPLE_PKG_NB] =
{
    NULL,

    "fun [S W] I",    /* prototype Scol de cette fonction : on notera que le nombre d'argument est bien 1 */
    "fun [Chn S S fun [ObjCurl u0 S I S S] u1 u0] ObjCurl",
    "fun [ObjCurl] I"
};


/** \brief Fonction défininssant l'API Scol de la bibliothèque.
 *
 * \param structure mmachine
 * \return 0 si succès
 *
 * Une API Scol est définie par 4 tableaux et une fonction.
 *
 * Les 4 tableaux ont tous une taille identique. Ceux-ci définissent respectivement le nom Scol des objets
 * de l'API, le nom des fonctions internes correspondantes, le nombre d'arguments ou le type d'objet,
 * et le prototypage Scol.
 *
 * La fonction interne 'PKhardpak' les chargera dans l'environnement minimal. Ses arguments sont
 * respectivement définis comme suit :
 * - la machine Scol correspondante
 * - le nom interne du package
 * - le nombre d'éléments de l'API (donc la taille de chaque tableau)
 * - les noms des éléments de votre API (fonctions, types, flags, ...)
 * - leurs correspondants internes
 * - le nombre d'argument qu'ils attendent (si cela a un sens, sinon ce sera NULL)
 * - le typage Scol
 *
 * Dans notre exemple, cette API est définie par 4 tableaux de EXAMPLE_PKG_NB cases précisément.
 * Il est impératif que les 4 tableaux aient le même nombre de cases et que cenombre soit strictement
 * équivalent au nombre de fonctions de l'API.
 *
 * Le comppilateur pourra ne rien dire si le nombre de cases est plus grand que le nombre réelle de
 * fonctions de l'API (le cas inverse engendrera un warning si vous avez suivi mes options de compilation).
 * Cependant, à l'exécution, la bibliothèque ne sera pas chargée et il n'y aura aucun message d'erreur.
 * Ŝoyez attentif à ce niveau (comme au reste !).
 * Notez que "nombre de fonctions" inclue les fonctions proprement dit mais aussi les objets Scol
 * (les types Scol), les flags, etc ... définis dans votre API.
 *
 * Les tableaux doivent correspondre, c'est-à-dire que le nom de l'élément de l'API situé à la
 * première case doit correspondre la fonction interne situé à la même case du second tableau, qui
 * doit correspondre au nombre d'argument situé à la même case du troisième tableau et même chose
 * pour le quatrième tableau.
 *
 * Enfin, il va de soi que le nombre d'arguments défini pour un élément de l'API dans le troisème
 * tableau doit être strictement cohérent avec le typage défini au quatrième tableau.
 *
 * Par exemple : si le nombre d'argument est 2, le typage de la fonction pourra être "fun [I S] I"
 * ou "fun [Chn S]" mais pas "fun [I S I]" ou "fun [S] I".
 * Là encore, aucune erreur ne peut être détecté lors de la compilation mais lorsque la tentative de
 * chargement de votre bibliothèque par le noyau échouera immanquablement sans la moindre erreur
 * apparente.
 */
int SCOLinitEXAMPLEclass (mmachine m)
{
    int k = 0;

    MMechostr (MSKDEBUG, "SCOLinitEXAMPLEclass library : loading\n");

    ObjCurl  = OBJregister (CURL_RFL_NB, 1, ObjCurlDestroy, "ObjCurlType");
    OBJgetUserEvent();

    k = PKhardpak (m, "EXAMPLEengine", EXAMPLE_PKG_NB, example_name, example_fun, example_narg, example_type);
    return k;
}





/** \fn SCOLloadEXAMPLE
 * \brief Fonction de chargement de la bibliothèque par le noyau.
 *
 * \param structure mmachine
 * \param structure cbmachine (uniquement pour la version MS Windows)
 * \return 0 si succès
 *
 * C'est donc cette fonction qui sera appelé par le noyau pour charger cette bibliothèque dans l'environnement
 * minimal de chaque VM Scol exécutée.
 * Le nom de cette fonction, ici 'SCOLloadEXAMPLE', devra être reportée dans la ligne adéquate du fichier usm.ini :
 *
 * # sous MS Windows :
 * \code plugin plugins/lib.example.dll SCOLloadEXAMPLE SCOLfreeEXAMPLE\endcode
 * # sous GNU / Linux :
 * \codeplugin plugins/lib.example.so SCOLloadEXAMPLE SCOLfreeEXAMPLE\endcode
 *
 * Cette fonction contient la fonction contenant API Scol ('SCOLinitEXAMPLEclass'), utilisable par n'importe quel
 * script Scol (du moment que cette librairie est chargée dans l'environnement minimal tel que décrit ci-dessus).
 *
 * Enfin, il est à noter que le chargement est différent sous MS Windows et GNU / Linux. On utilise donc le
 * preprocesseur pour aiguiller le bon chargement selon la plateforme utilisée lors de la compilation.
*/
#if defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLloadEXAMPLE (mmachine m, cbmachine w)
{
    int k = 0;
    /* Affectation aux variables globales ww et mm des valeurs envoyées par le noyau des structures partagées.
    Il est impératif de les ajouter AVANT tout appel à des fonctions ou macros Scol sous peine de crash immédiat
    lors de l'exécution de Scol. */
    ww = w;
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}


#elif defined linux || defined __linux
int SCOLloadEXAMPLE (mmachine m)
{
    int k = 0;
    /* Affectation à la variable globale mm des valeurs envoyées par le noyau des structures partagées.
    Il est impératif de l'ajouter AVANT tout appel à des fonctions ou macros Scol sous peine de crash immédiat
    lors de l'exécution de Scol. */
    mm = m;

    MMechostr (0, "EXAMPLE library : loading\n");

    if ((k = SCOLinitEXAMPLEclass (m))) return k;

    MMechostr (0, "\nEXAMPLE library loaded !\n");
    return k;
}

/* Autres plateformes (le compilateur retournera l'erreur et s'arrêtera) */
#else
#error no platform supported
#endif



/** \brief Fonction de déchargement / libération
 *
 * \return 0 si succès
 *
 * La fonction est appelée lorsque Scol se termine. Elle définit, comme pour la fonction de chargement,
 * dans le fichier usm.ini (voir ci-dessus).
 *
 * Les éventuels objets Scol présents devront avoir été détruits au préalable. Ici, on peut, par exemple,
 * libérer des ressources tierces telle qu'une bibliothèque
*/
#if defined linux || defined __linux
int SCOLfreeEXAMPLE ()

#elif defined _WIN32 || defined __WIN32__
__declspec (dllexport) int SCOLfreeEXAMPLE ()

#else
#error no platform supported
#endif

{
    MMechostr(MSKDEBUG, "\nEXAMPLE library release !\n");
    return 0;
}
